@@ -31,7 +31,7 @@ use List::MoreUtils qw(any none);
 use Text::ParseWords ();
 
 use Lintian::Spelling qw(check_spelling);
-use Lintian::Util qw(clean_env do_fork drain_pipe internal_error open_gz);
+use Lintian::Util qw(clean_env do_fork drain_pipe open_gz);
 
 use constant LINTIAN_COVERAGE => ($ENV{'LINTIAN_COVERAGE'}?1:0);
 use constant EMPTY => q{};
@@ -59,6 +59,10 @@ my @admin_locations= qw(sbin/ usr/sbin/);
 sub files {
     my ($self, $file) = @_;
 
+    # no man pages in udebs
+    return
+      if $self->type eq 'udeb';
+
     if ($file->name =~ m,^usr/share/man/\S+,) {
 
         $self->tag('manpage-in-udeb', $file->name)
@@ -168,12 +172,12 @@ sub files {
             }
         }
     } else { # not a symlink
-        my $fs_path = $file->fs_path;
+        my $unpacked_path = $file->unpacked_path;
         my $fd;
         if ($file_info =~ m/gzip compressed/) {
-            $fd = $file->open_gz;
+            $fd = open_gz($file->unpacked_path);
         } else {
-            $fd = $file->open;
+            open($fd, '<', $file->unpacked_path);
         }
         my @manfile = <$fd>;
         close $fd;
@@ -226,8 +230,8 @@ sub files {
             if ($pid == 0) {
                 clean_env;
                 open(STDERR, '>&', \*STDOUT);
-                exec('lexgrog', $fs_path)
-                  or internal_error("exec lexgrog failed: $!");
+                exec('lexgrog', $unpacked_path)
+                  or die "exec lexgrog failed: $!";
             }
             if (@{$self->running_lexgrog} > 2) {
                 $self->process_lexgrog_output($self->running_lexgrog);
@@ -244,11 +248,11 @@ sub files {
         # pages with .so but aren't simple links; rbash, for instance.)
         my @cmd = qw(man --warnings -E UTF-8 -l -Tutf8 -Z);
         my $dir;
-        if ($fs_path =~ m,^(.*)/(man\d/.*)$,) {
+        if ($unpacked_path =~ m,^(.*)/(man\d/.*)$,) {
             $dir = $1;
             push @cmd, $2;
         } else {
-            push(@cmd, $fs_path);
+            push(@cmd, $unpacked_path);
         }
         my ($read, $write);
         pipe($read, $write);
@@ -265,7 +269,7 @@ sub files {
             $ENV{MANROFFSEQ} = '';
             $ENV{MANWIDTH} = 80;
             exec { $cmd[0] } @cmd
-              or internal_error("cannot run man -E UTF-8 -l: $!");
+              or die "cannot run man -E UTF-8 -l: $!";
         } else {
             # parent - close write end
             close $write;
@@ -316,10 +320,14 @@ sub files {
 sub breakdown {
     my ($self) = @_;
 
+    # no man pages in udebs
+    return
+      if $self->type eq 'udeb';
+
     my %local_user_executables;
     my %local_admin_executables;
 
-    for my $file ($self->processable->sorted_index) {
+    for my $file ($self->processable->installed->sorted_list) {
 
         next
           unless $file->is_symlink || $file->is_file;
@@ -338,7 +346,7 @@ sub breakdown {
 
     my @direct_reliants
       =@{$self->group->direct_reliants($self->processable) // []};
-    my@reliant_files = map { $_->sorted_index } @direct_reliants;
+    my@reliant_files = map { $_->installed->sorted_list } @direct_reliants;
 
     # for executables, look at packages relying on the current processable
     my %distant_executables;
@@ -358,7 +366,8 @@ sub breakdown {
 
     my @direct_prerequisites
       =@{$self->group->direct_dependencies($self->processable) // []};
-    my@prerequisite_files = map { $_->sorted_index } @direct_prerequisites;
+    my@prerequisite_files
+      = map { $_->installed->sorted_list } @direct_prerequisites;
 
     # for manpages, look at packages the current processable relies upon
     my %distant_manpages;
@@ -404,9 +413,10 @@ sub breakdown {
         my $file = $local_admin_executables{$command};
         my @manpages = @{$related_manpages{$command} // []};
 
+        my @sections = grep { defined } map { $_->{section} } @manpages;
         $self->tag('command-in-sbin-has-manpage-in-incorrect-section', $file)
           if $file->is_regular_file
-          && any { $_->{section} == 1 } @manpages;
+          && any { $_ == 1 } @sections;
     }
 
     $self->tag('binary-without-english-manpage', $_)
@@ -451,7 +461,8 @@ sub process_lexgrog_output {
         eval {close($lexgrog_fd);};
         if (my $err = $@) {
             # Problem closing the pipe?
-            internal_error("close pipe: $err") if $err->errno;
+            die "close pipe: $err: $!"
+              if $err->errno;
             # No, then lexgrog returned with a non-zero exit code.
             $self->tag('manpage-has-bad-whatis-entry', $file);
         }
