@@ -36,7 +36,7 @@ has arch_dep_files => (is => 'rwp', default => 0);
 sub files {
     my ($self, $file) = @_;
 
-    my $architecture = $self->info->field('architecture', '');
+    my $architecture = $self->processable->field('architecture', '');
 
     if ($file->name =~ m,^(?:usr/)?lib/([^/]+)/$,o) {
         my $subdir = $1;
@@ -65,10 +65,10 @@ sub files {
 sub breakdown {
     my ($self) = @_;
 
-    my $architecture = $self->info->field('architecture', '');
+    my $architecture = $self->processable->field('architecture', '');
 
     # check if package is empty
-    my $is_dummy = $self->info->is_pkg_class('any-meta');
+    my $is_dummy = $self->processable->is_pkg_class('any-meta');
 
     $self->tag('package-contains-no-arch-dependent-files')
       unless $is_dummy
